@@ -26,6 +26,8 @@
 #include <QStandardPaths>
 #include <QJsonArray>
 #include <QRegularExpression>
+#include <QDBusConnectionInterface>
+#include <QDBusConnection>
 
 #include <KPluginLoader>
 #include <KPluginMetaData>
@@ -50,7 +52,8 @@ static bool defaultMatch(const QString& constraint, const QJsonValue& value)
 static bool mimeTypeMatch(const QString& constraint, const QJsonValue& value)
 {
     if(value.isArray()) {
-        foreach(const QJsonValue& val, value.toArray()) {
+        const auto array = value.toArray();
+        for (const QJsonValue& val : array) {
             if (mimeTypeMatch(constraint, val))
                 return true;
         }
@@ -70,8 +73,29 @@ static bool mimeTypeMatch(const QString& constraint, const QJsonValue& value)
     }
 }
 
+static bool dbusMatch(const QString& constraint, const QJsonValue& value)
+{
+    Q_UNUSED(value)
+    return QDBusConnection::sessionBus().interface()->isServiceRegistered(constraint);
+}
+
+static bool executablePresent(const QString& constraint, const QJsonValue& value)
+{
+    Q_UNUSED(value)
+    return !QStandardPaths::findExecutable(constraint).isEmpty();
+}
+
+static bool desktopFilePresent(const QString& constraint, const QJsonValue& value)
+{
+    Q_UNUSED(value)
+    return !QStandardPaths::locate(QStandardPaths::ApplicationsLocation, constraint).isEmpty();
+}
+
 static QMap<QString, matchFunction> s_matchFunctions = {
-    { QStringLiteral("mimeType"), mimeTypeMatch }
+    { QStringLiteral("mimeType"), mimeTypeMatch },
+    { QStringLiteral("dbus"), dbusMatch },
+    { QStringLiteral("application"), desktopFilePresent },
+    { QStringLiteral("exec"), executablePresent }
 };
 
 class Purpose::AlternativesModelPrivate
@@ -82,11 +106,13 @@ public:
     QString m_pluginType;
     QStringList m_disabledPlugins = s_defaultDisabledPlugins;
     QJsonObject m_pluginTypeData;
+    const QRegularExpression constraintRx { QStringLiteral("(\\w+):(.*)") };
 
-    bool isPluginAcceptable(const KPluginMetaData &meta, const QStringList &disabledPlugins) const {
+    bool isPluginAcceptable(const KPluginMetaData &meta, const QStringList &disabledPlugins) const
+    {
         const QJsonObject obj = meta.rawData();
         if(!obj.value(QStringLiteral("X-Purpose-PluginTypes")).toArray().contains(m_pluginType)) {
-            qDebug() << "discarding" << meta.name() << meta.value(QStringLiteral("X-Purpose-PluginTypes"));
+            qDebug() << "discarding" << meta.name() << KPluginMetaData::readStringList(meta.rawData(), QStringLiteral("X-Purpose-PluginTypes"));
             return false;
         }
 
@@ -95,25 +121,41 @@ public:
             return false;
         }
 
+        //All constraints must match
         const QJsonArray constraints = obj.value(QStringLiteral("X-Purpose-Constraints")).toArray();
-        const QRegularExpression constraintRx(QStringLiteral("(\\w+):(.*)"));
         for(const QJsonValue& constraint: constraints) {
-            Q_ASSERT(constraintRx.isValid());
-            QRegularExpressionMatch match = constraintRx.match(constraint.toString());
-            if (!match.isValid() || !match.hasMatch()) {
-                qWarning() << "wrong constraint" << constraint.toString();
-                continue;
-            }
-            QString propertyName = match.captured(1);
-            QString constrainedValue = match.captured(2);
-            bool acceptable = s_matchFunctions.value(propertyName, defaultMatch)(constrainedValue, m_inputData.value(propertyName));
-            if (!acceptable) {
-//                 qDebug() << "not accepted" << meta.name() << propertyName << constrainedValue << m_inputData[propertyName];
+            if (!constraintMatches(meta, constraint))
                 return false;
-            }
         }
         return true;
     }
+
+    bool constraintMatches(const KPluginMetaData &meta, const QJsonValue &constraint) const
+    {
+        //Treat an array as an OR
+        if (constraint.isArray()) {
+            const QJsonArray options = constraint.toArray();
+            for (const auto &option: options) {
+                if (constraintMatches(meta, option)) {
+                    return true;
+                }
+            }
+            return false;
+        }
+        Q_ASSERT(constraintRx.isValid());
+        QRegularExpressionMatch match = constraintRx.match(constraint.toString());
+        if (!match.isValid() || !match.hasMatch()) {
+            qWarning() << "wrong constraint" << constraint.toString();
+            return false;
+        }
+        const QString propertyName = match.captured(1);
+        const QString constrainedValue = match.captured(2);
+        const bool acceptable = s_matchFunctions.value(propertyName, defaultMatch)(constrainedValue, m_inputData.value(propertyName));
+        if (!acceptable) {
+//             qDebug() << "not accepted" << meta.name() << propertyName << constrainedValue << m_inputData[propertyName];
+        }
+        return acceptable;
+    }
 };
 
 AlternativesModel::AlternativesModel(QObject* parent)
@@ -133,7 +175,8 @@ QHash<int,QByteArray> AlternativesModel::roleNames() const
     QHash<int,QByteArray> roles = QAbstractListModel::roleNames();
     roles.unite({
         { IconNameRole, "iconName" },
-        { PluginIdRole, "pluginId" }
+        { PluginIdRole, "pluginId" },
+        { ActionDisplayRole, "actionDisplay" }
     });
     return roles;
 }
@@ -228,6 +271,10 @@ QVariant AlternativesModel::data(const QModelIndex& index, int role) const
             return QIcon::fromTheme(data.iconName());
         case PluginIdRole:
             return data.pluginId();
+        case ActionDisplayRole: {
+            QString action = data.rawData()[QStringLiteral("KPlugin")].toObject()[QStringLiteral("X-Purpose-ActionDisplay")].toString();
+            return action.isEmpty() ? data.name() : action;
+        }
     }
     return QVariant();
 }
@@ -235,8 +282,9 @@ QVariant AlternativesModel::data(const QModelIndex& index, int role) const
 static QVector<KPluginMetaData> findScriptedPackages(std::function<bool(const KPluginMetaData &)> filter)
 {
     QVector<KPluginMetaData> ret;
+    QSet<QString> addedPlugins;
     const QStringList dirs = QStandardPaths::locateAll(QStandardPaths::GenericDataLocation, QStringLiteral("kpackage/Purpose"), QStandardPaths::LocateDirectory);
-    foreach(const QString &dir, dirs) {
+    for (const QString &dir : dirs) {
         QDirIterator dirIt(dir, QDir::Dirs | QDir::NoDotAndDotDot);
 
         for(; dirIt.hasNext(); ) {
@@ -246,7 +294,8 @@ static QVector<KPluginMetaData> findScriptedPackages(std::function<bool(const KP
                 continue;
 
             const KPluginMetaData info = Purpose::createMetaData(dir.absoluteFilePath(QStringLiteral("metadata.json")));
-            if (filter(info)) {
+            if (!addedPlugins.contains(info.pluginId()) && filter(info)) {
+                addedPlugins << info.pluginId();
                 ret += info;
             }
         }
@@ -263,7 +312,7 @@ void AlternativesModel::initializeModel()
     }
 
     const QJsonArray inbound = d->m_pluginTypeData.value(QStringLiteral("X-Purpose-InboundArguments")).toArray();
-    foreach(const QJsonValue& arg, inbound) {
+    for (const QJsonValue& arg : inbound) {
         if(!d->m_inputData.contains(arg.toString())) {
             qWarning() << "Cannot initialize model with data" << d->m_inputData << ". missing:" << arg;
             return;
@@ -278,7 +327,15 @@ void AlternativesModel::initializeModel()
     };
 
     beginResetModel();
-    d->m_plugins = KPluginLoader::findPlugins(QStringLiteral("purpose"), pluginAcceptable);
+    d->m_plugins.clear();
+    const auto plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/purpose"));
+    QSet<QString> addedPlugins;
+    for (const auto &metaData : plugins) {
+        if (!addedPlugins.contains(metaData.pluginId()) && pluginAcceptable(metaData)) {
+            addedPlugins << metaData.pluginId();
+            d->m_plugins << metaData;
+        }
+    }
     d->m_plugins += findScriptedPackages(pluginAcceptable);
     endResetModel();
 }
